package rec;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * This is the main class of the recommender package.
 * It reads the parameter file, runs the specified predictors and computes the predictive errors.
 * All parameters should be specified using the parameterFile "parameters.txt" 
 */
public class Recommender {

	//default settings
	private static int neighbourhoodSize = 20;
	private static String fileDirectory = "ml-100k/";
	private static String trainDataFile = "ua.base";
	private static String testDataFile = "ua.test";
	private static String parameterFile = "src/rec/parameters.txt";
	private static String[] predictors = {"AverageBased"};

	private static final boolean DEBUG = true;
	private static boolean logLevel = !DEBUG;



	/**
	 * The main function sets the parameters, runs read the training and test data and runs the predictors.
	 * @param args (not needed)
	 */
	public static void main(String[] args) {
		//set the parameters. unless the parameter is mentioned in the text file, the default is set
		ParseInput.setParameters(parameterFile);

		//read-in training and test data
		int[][] trainData = readData(fileDirectory+trainDataFile);
		int[][] testData = readData(fileDirectory+testDataFile);
		

		//compute the predictions and errors for all algorithms listed
		double [] predictiveErrors = new double[predictors.length];
		double [] runTimes = new double[predictors.length];
		
		for (int i = 0; i<predictors.length; i++){
			runTimes[i] = System.nanoTime();
			predictiveErrors[i] = runTest(predictors[i],trainData,testData);
			runTimes[i] = (System.nanoTime()- runTimes[i])/Math.pow(10, 9);
		}
		
		//display results
		System.out.println("========================================");
		System.out.println("Results");
		System.out.println("========================================");
		
		System.out.println("Predictor \t\t RMSE \t Run Time (s)\n");
		for (int i = 0; i<predictors.length; i++){
			System.out.println(String.format("%s \t\t %.3f \t %.3f\n", predictors[i], predictiveErrors[i], runTimes[i]));
		}
	}

	/**
	 * Runs the predictor specified and computes the root mean squared error
	 * @param predictor: name of the predictor: averagebased, useraverage, userbased, itembased, mypredictor
	 * @param trainData: data used to train the predictor
	 * @param testData: data used to test the accuracy of the predictions
	 * @return root mean square error of the predictions
	 */
	public static double runTest(String predictor, int[][] trainData, int[][] testData){

		Predictor p = null;
		
		if (predictor.toLowerCase().equals("averagebased")){
			p = new AverageBasedPredictor();
		}
		else if (predictor.toLowerCase().equals("useraverage")){
			p = new UserAverageBasedPredictor();
		}
		else if (predictor.toLowerCase().equals("userbased")){
			p = new UserBasedPredictor(neighbourhoodSize);
		}
		else if (predictor.toLowerCase().equals("itembased")){
			p = new ItemBasedPredictor(neighbourhoodSize);
		}
		else if (predictor.toLowerCase().equals("mypredictor")){
			p = new MyPredictor();
		}

		//train the predictor
		p.train(trainData);

		//compute the predictions
		int N = testData.length;
		double[] predictions = new double[N];
		for (int i = 0; i<N; i++){
			predictions[i] = p.predict(testData[i][0], testData[i][1]);
		}

		if (logLevel == DEBUG){
			System.out.println("========================================");
			System.out.println("Predictions from "+predictor);
			System.out.println("========================================");
			System.out.println("User \t Movie \t Rating  Prediction");
			for (int i = 0; i<N; i++){
				System.out.println((String.format("%d \t %d \t %d \t %.3f \n",testData[i][0],testData[i][1],testData[i][2],predictions[i] )));
			}
		}

		//compute the root mean squared error
		double RMSE = 0;
		for (int i = 0; i<N;i++){
			RMSE += (predictions[i]-testData[i][2])*(predictions[i]-testData[i][2]);
		}
		RMSE = Math.sqrt(RMSE/(double)N);

		return RMSE;
	}

	/**
	 * Reads in data from a file and converts them into an integer array
	 * Data are expected to be of the form (user, movie, rating)_i 
	 * @param fileName: the data file
	 * @return an integer array (user, movie, rating)_i
	 */
	public static int[][] readData(String fileName){
		BufferedReader b;
		int [][] data = null;
		try {
			b = new BufferedReader(new FileReader(fileName));
			String line;
			//count number of lines
			int N = 0;
			while( (line = b.readLine()) != null){
				N++;
			}
			b.close();

			//data is of the form: (userID, movieID,rating)_i
			data = new int[N][3];
			int count = 0;
			b = new BufferedReader(new FileReader(fileName));
			while( (line = b.readLine()) != null){
				String[] s = line.split("\t");
				for (int i = 0; i<3; i++){
					try {
						data[count][i] = new Integer(s[i]);
					}
					catch (NumberFormatException nfe){
						System.err.println("input data must be of type integer.");
						nfe.printStackTrace();
					}

				}
				count++;
			}
			b.close();

		}
		catch (FileNotFoundException e) {
			System.err.println("Couldn't read the parameter file. Check the file name: "+fileName);
			e.printStackTrace();
		} 
		catch(IOException ioe){
			ioe.printStackTrace();
		}

		return data;
	}


	public static int getNeighbourhoodSize() {
		return neighbourhoodSize;
	}

	public static void setNeighbourhoodSize(int neighbourhoodSize) {
		Recommender.neighbourhoodSize = neighbourhoodSize;
	}

	public static void setLoglevel(String log){
		if (log.toLowerCase().equals("default")){
			logLevel = !DEBUG;
		}
		else if (log.toLowerCase().equals("debug")){
			logLevel = DEBUG;
		}
	}

	public static void setTrainDataFile(String trainDataFile) {
		Recommender.trainDataFile = trainDataFile;
	}

	public static void setTestDataFile(String testDataFile) {
		Recommender.testDataFile = testDataFile;
	}

	public static void setPredictors(String[] predictors) {
		Recommender.predictors = predictors;
	}

	public static String getTrainDataFile() {
		return trainDataFile;
	}

	public static String getTestDataFile() {
		return testDataFile;
	}

	public static String[] getPredictors() {
		return predictors;
	}

	public static boolean getLogLevel() {
		return logLevel;
	}

	



}
